package com.offcn.live.tsdm;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.os.Environment;
import android.support.annotation.NonNull;
import android.text.TextUtils;

import com.jyall.base.log.LogUtils;
import com.jyall.base.util.CommonUtils;
import com.jyall.base.util.ThreadPoolUtils;
import com.jyall.base.util.ValidateUtils;
import com.maning.updatelibrary.http.AbsFileProgressCallback;
import com.maning.updatelibrary.http.DownloadFileUtils;
import com.offcn.live.tsdm.api.RetrofitManager;
import com.offcn.live.tsdm.api.network.ProgressSubscriber;
import com.offcn.live.tsdm.bean.ZGLDownLoaderBean;
import com.offcn.live.tsdm.bean.ZGLDownLoaderFileBean;
import com.offcn.live.tsdm.bean.ZGLDownloadStatusEnum;
import com.offcn.live.tsdm.bean.ZGLPlayBackVideoBean;
import com.offcn.live.tsdm.bean.ZGLPlaybackBean;
import com.offcn.live.tsdm.bean.ZGLSignInBean;
import com.offcn.live.tsdm.greendao.gen.DaoMaster;
import com.offcn.live.tsdm.greendao.gen.DaoSession;
import com.offcn.live.tsdm.greendao.gen.ZGLDownLoaderBeanDao;
import com.offcn.live.tsdm.greendao.gen.ZGLDownLoaderFileBeanDao;
import com.offcn.live.tsdm.util.ZGLUtils;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.schedulers.Schedulers;
import okhttp3.ResponseBody;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import tsdm.live.offcn.com.tsdownloadermanager.R;


public class ZGLDownLoaderManager {

    private static final String TAG = ZGLDownLoaderManager.class.getSimpleName();

    private static final String DOWNLOAD_PATH = Environment.getExternalStorageDirectory() + "/zgl/";

    public static final String NAME_DB = "local.db";
    public static final String NAME_M3U8 = "local.m3u8";

    private static volatile ZGLDownLoaderManager instance;

    private Context mContext;

    // 来自外部的扩展参数，通过接口透传到服务器
    private Map<String, String> mIntentExtensionsMap;

    // 正在下载的队列
    private List<String> mCurDownloadingList = new ArrayList<>();
    // 暂停中队列
    private List<String> mCurPausedList = new ArrayList<>();
    // 等待中队列
    private List<String> mCurWaitingList = new ArrayList<>();
    // 准备中队列
    private List<String> mCurPreparedList = new ArrayList<>();


    // 集合存储code与对应信息：对应的文件列表集合，对应的文件总数
    private Map<String, List<ZGLDownLoaderFileBean>> mDownloadInfoMap = new HashMap<>();


    // 外部传递来的下载存储目录
    private String mIntentDirPath = DOWNLOAD_PATH;
    // 从外部传递来的hook参数, 自己的服务器用
    private String mIntentHook;

    // 获取当前code对应的目录
    private String getDownloadPath(String code) {
        return mIntentDirPath + code + "/";
    }

    // 获取当前code对应的本地m3u8文件地址
    public String getDownloadFilePath(String code) {
        return getDownloadPath(code) + NAME_M3U8;
    }

    // 获取当前code对应的本地m3u8文件地址
    public String getDownloadDBPath(String code) {
        return getDownloadPath(code) + NAME_DB;
    }


    // 最大同时下载数,默认为1个
    private int mMaxDownloadNum = 1;

    public ZGLDownLoaderManager maxNum(int maxDownloadNum) {
        if (maxDownloadNum > 0) {
            this.mMaxDownloadNum = maxDownloadNum;
        }
        return this;
    }

    private Context getActivity() {
        return mContext;
    }

    private ZGLDownLoaderManager(Context context) {
        this.mContext = context.getApplicationContext();

        this.mIntentHook = "";

        initDB();
    }

    public static ZGLDownLoaderManager getInstance(Context context) {
        if (instance == null) {
            synchronized (ZGLDownLoaderManager.class) {
                if (instance == null) {
                    instance = new ZGLDownLoaderManager(context);
                }
            }
        }

        return instance;
    }

    // DB相关
    private DaoMaster.DevOpenHelper mHelper;
    private SQLiteDatabase db;
    private DaoMaster mDaoMaster;
    private DaoSession mDaoSession;
    private ZGLDownLoaderBeanDao mBeanDao;
    private ZGLDownLoaderFileBeanDao mFileBeanDao;

    private void initDB() {
        // 通过 DaoMaster 的内部类 DevOpenHelper，你可以得到一个便利的 SQLiteOpenHelper 对象。
        // 可能你已经注意到了，你并不需要去编写「CREATE TABLE」这样的 SQL 语句，因为 greenDAO 已经帮你做了。
        // 注意：默认的 DaoMaster.DevOpenHelper 会在数据库升级时，删除所有的表，意味着这将导致数据的丢失。
        // 所以，在正式的项目中，你还应该做一层封装，来实现数据库的安全升级。
        mHelper = new DaoMaster.DevOpenHelper(getActivity(), "zgl-video-db", null);
        db = mHelper.getWritableDatabase();
        // 注意：该数据库连接属于 DaoMaster，所以多个 Session 指的是相同的数据库连接。
        mDaoMaster = new DaoMaster(db);
        mDaoSession = mDaoMaster.newSession();
        mBeanDao = mDaoSession.getZGLDownLoaderBeanDao();
        mFileBeanDao = mDaoSession.getZGLDownLoaderFileBeanDao();


        File file = new File(DOWNLOAD_PATH);
        if (!file.exists()) {
            file.mkdirs();
        }
    }

    /**
     * 扩展参数，用于第三方平台的验证
     *
     * @param map
     * @return
     */
    public ZGLDownLoaderManager extensions(Map<String, String> map) {
        this.mIntentExtensionsMap = map;
        return this;
    }

    /**
     * hook回调
     *
     * @param hook
     * @return
     */
    public ZGLDownLoaderManager hook(String hook) {
        this.mIntentHook = hook;
        return this;
    }

    /**
     * 存储目录
     *
     * @param downloadPath
     */
    public ZGLDownLoaderManager downloadPath(String downloadPath) {
        if (TextUtils.isEmpty(downloadPath)) {
            downloadPath = DOWNLOAD_PATH;
        }

        File dirFile = new File(downloadPath);

        // 若路径为文件，则抛异常
        if (dirFile.isFile()) {
            throw new InvalidParameterException("downloadPath must be dir NOT file");
        }
        if (dirFile.isDirectory() && !dirFile.exists()) {
            // 若创建目录失败，则抛异常
            if (!dirFile.mkdirs()) {
                throw new InvalidParameterException("downloadPath check if valid");
            }
        }

        // 若传递来的目录不以 / 结尾，则手动添加上
        if (!downloadPath.endsWith("/")) {
            downloadPath += "/";
        }
        mIntentDirPath = downloadPath;

        return this;
    }

    /**
     * 开启下载。不需要 额外id
     *
     * @param code
     */
    public void start(@NonNull final String code) {
        start(code, "");
    }

    /**
     * 开始下载的对外暴露方法
     *
     * @param code        房间口令
     * @param extensionId 外部的id，目前主要兼容在线课堂的特殊要求
     */
    public void start(@NonNull final String code, final String extensionId) {
        LogUtils.e(TAG, "调用 start 方法");

        if (getActivity() == null) {
            return;
        }

        if (TextUtils.isEmpty(code)) {
            CommonUtils.showToast(getActivity(), "请设置房间口令");
            return;
        }

        if (mOnTsProgressListener == null) {
            CommonUtils.showToast(getActivity(), "请设置回调监听");
            return;
        }

        checkMaxNum(code, extensionId);

        // 如果正在下载集合中存在此code，则停止
        if (mCurDownloadingList.contains(code) || mCurDownloadingList.contains(extensionId)) {
            return;
        }

        if (!CommonUtils.isNetConnected(getActivity())) {
            CommonUtils.showToast(getActivity(), R.string.net_off);
            return;
        }

        // 开启内部下载方法
        startInner(code, extensionId);

    }

    /**
     * 开始下载的内部逻辑
     *
     * @param roomCode
     */
    private void startInner(@NonNull final String roomCode, final String extensionId) {
        LogUtils.e(TAG, "调用 startInner 方法");


        // 回调方法：准备中-接口获取文件和写入
        if (mOnTsProgressListener != null) {
            mOnTsProgressListener.onPrepared(roomCode, extensionId);
        }

        // 检查当前下载是否超过最大
        checkMaxNum(roomCode, extensionId);

        // 若DB中存在此code，则直接开始下载
        List<ZGLDownLoaderBean> tmpList = mBeanDao.loadAll();
        if (tmpList != null && tmpList.size() != 0) {
            for (ZGLDownLoaderBean bean : tmpList) {
                if (roomCode.equals(bean.getCode())) {
                    // 存在此code
                    if (bean.isDownloaded()) {
                        LogUtils.e(TAG, "DB存储为下载完成");
                        // 已下载完毕
                        if (bean.isLocalM3u8Exist()) {
                            LogUtils.e(TAG, "SD卡存在此口令文件");
                            // 存在对应文件 local.m3u8
                            if (mOnTsProgressListener != null) {
                                mOnTsProgressListener.onSuccess(roomCode, extensionId);
                            }
                        } else {
                            LogUtils.e(TAG, "SD卡不存在此口令文件");
                            // 若文件不存在 local.m3u8
                            delete(roomCode, null);
                            delete(extensionId, null);
                            start(roomCode, extensionId);
                        }
                    } else {
                        LogUtils.e(TAG, "DB存储为未下载完成");
                        resumeTask(roomCode, extensionId);
                    }

                    return;
                }

            }
        }

        // 先判断有没有回放视频资源，有再去下载，没有则提示
        RetrofitManager.getInstance(getActivity())
                .getPlaybackExist(roomCode)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(new ProgressSubscriber<ZGLSignInBean>(getActivity()) {
                    @Override
                    public void onResponse(ZGLSignInBean result) {
                        if (!result.active) {
                            // 没有回放视频，则直接报错提示
                            LogUtils.e(TAG, "没有可下载资源 " + roomCode);

                            if (null != mOnTsProgressListener) {
                                mOnTsProgressListener.onFailed(roomCode, extensionId, "当前没有可下载资源");
                            }
                        } else {
                            // 有回放视频资源，则开启下载
                            Map<String, String> paramsMap = new HashMap<>();
                            paramsMap.put("account", roomCode);
                            if (!ValidateUtils.isEmpty(mIntentHook)) {
                                paramsMap.put("hook", mIntentHook);
                            }
                            if (mIntentExtensionsMap != null && mIntentExtensionsMap.size() > 0) {
                                paramsMap.putAll(mIntentExtensionsMap);
                            }

                            RetrofitManager.getInstance(getActivity())
                                    .getPlaybackInfo(paramsMap)
                                    .subscribeOn(Schedulers.io())
                                    .observeOn(AndroidSchedulers.mainThread())
                                    .subscribe(new ProgressSubscriber<ZGLPlaybackBean>(getActivity()) {
                                        @Override
                                        public void onResponse(final ZGLPlaybackBean result) {
                                            final String url = result.getServerUrl();
//                        final String url = "http://service-xiaoyu.oss-cn-beijing.aliyuncs.com/vod/8916738ed64ff4bd1159e41ba5e39aec/index.m3u8";


                                            // local.m3u8
                                            Call<ResponseBody> callLocal = RetrofitManager.getInstance(getActivity())
                                                    .getPlaybackInfoFile(result.getLocalUrl());
                                            callLocal.enqueue(new Callback<ResponseBody>() {
                                                @Override
                                                public void onResponse(Call<ResponseBody> call, Response<ResponseBody> response) {

                                                    String dirPath = getDownloadPath(roomCode);

                                                    File dirFile = new File(dirPath);

                                                    if (!dirFile.exists()) {
                                                        dirFile.mkdirs();
                                                    }

                                                    // 分别声明ts带与不带http的文件地址
                                                    String fileLocalPath = dirPath + NAME_M3U8;
                                                    File localFile = new File(fileLocalPath);

                                                    if (!localFile.exists()) {
                                                        try {
                                                            localFile.createNewFile();
                                                        } catch (IOException e) {
                                                            e.printStackTrace();
                                                        }
                                                    }

                                                    // 保存ts地址带与不带http的文件，只有两个都保存成功，才能下载
                                                    if (ZGLUtils.writeResponseBodyToDisk(localFile, response.body())) {
                                                        // 文件保存成功
                                                        // 读取文件内容，进行下载

                                                    } else {
                                                        // 文件保存失败

                                                        if (mOnTsProgressListener != null) {
                                                            mOnTsProgressListener.onFailed(roomCode, extensionId, "文件保存失败");
                                                        }
                                                    }
                                                }

                                                @Override
                                                public void onFailure(Call<ResponseBody> call, Throwable t) {
                                                    // 接口请求失败
                                                    if (mOnTsProgressListener != null) {
                                                        mOnTsProgressListener.onFailed(roomCode, extensionId, "接口请求失败");
                                                    }
                                                }
                                            });


                                            // index.m3u8
                                            Call<ResponseBody> call = RetrofitManager.getInstance(getActivity())
                                                    .getPlaybackInfoFile(url);
                                            call.enqueue(new Callback<ResponseBody>() {
                                                @Override
                                                public void onResponse(Call<ResponseBody> call, Response<ResponseBody> response) {

                                                    String dirPath = getDownloadPath(roomCode);

                                                    File dirFile = new File(dirPath);

                                                    if (!dirFile.exists()) {
                                                        dirFile.mkdirs();
                                                    }

                                                    // 分别声明ts带与不带http的文件地址
                                                    String fileServerPath = dirPath + "index.m3u8";
                                                    File serverFile = new File(fileServerPath);

                                                    if (!serverFile.exists()) {
                                                        try {
                                                            serverFile.createNewFile();
                                                        } catch (IOException e) {
                                                            e.printStackTrace();
                                                        }
                                                    }

                                                    // 保存ts地址带与不带http的文件，只有两个都保存成功，才能下载
                                                    if (ZGLUtils.writeResponseBodyToDisk(serverFile, response.body())) {
                                                        // 文件保存成功
                                                        // 读取文件内容，进行下载

                                                        addNewVideo(result.room_name, extensionId, processBody(roomCode, url, serverFile));

                                                    } else {
                                                        // 文件保存失败

                                                        if (mOnTsProgressListener != null) {
                                                            mOnTsProgressListener.onFailed(roomCode, extensionId, "文件保存失败");
                                                        }
                                                    }
                                                }

                                                @Override
                                                public void onFailure(Call<ResponseBody> call, Throwable t) {
                                                    // 接口请求失败
                                                    if (mOnTsProgressListener != null) {
                                                        mOnTsProgressListener.onFailed(roomCode, extensionId, "接口请求失败");
                                                    }
                                                }
                                            });


                                            // sqlite.db
                                            Call<ResponseBody> callSqlite = RetrofitManager.getInstance(getActivity())
                                                    .getPlaybackInfoFile(result.getSqliteUrl());
                                            callSqlite.enqueue(new Callback<ResponseBody>() {
                                                @Override
                                                public void onResponse(Call<ResponseBody> call, Response<ResponseBody> response) {

                                                    String dirPath = getDownloadPath(roomCode);

                                                    File dirFile = new File(dirPath);

                                                    if (!dirFile.exists()) {
                                                        dirFile.mkdirs();
                                                    }

                                                    // 声明DB存储路径
                                                    String fileSqlitePath = dirPath + NAME_DB;
                                                    File sqliteFile = new File(fileSqlitePath);

                                                    if (!sqliteFile.exists()) {
                                                        try {
                                                            sqliteFile.createNewFile();
                                                        } catch (IOException e) {
                                                            e.printStackTrace();
                                                        }
                                                    }

                                                    // 保存DB文件
                                                    if (ZGLUtils.writeResponseBodyToDisk(sqliteFile, response.body())) {
                                                        // 文件保存成功

                                                    } else {
                                                        // 文件保存失败
//                                                        mPreparedQueue.remove(roomCode);
//                                                        mPreparedQueue.remove(extensionId);
//                                                        if (mOnTsProgressListener != null) {
//                                                            mOnTsProgressListener.onFailed(roomCode, extensionId, "文件保存失败");
//                                                        }
                                                    }
                                                }

                                                @Override
                                                public void onFailure(Call<ResponseBody> call, Throwable t) {
                                                    // 接口请求失败
//                                                    mPreparedQueue.remove(roomCode);
//                                                    mPreparedQueue.remove(extensionId);
//                                                    if (mOnTsProgressListener != null) {
//                                                        mOnTsProgressListener.onFailed(roomCode, extensionId, "接口请求失败");
//                                                    }
                                                }
                                            });

                                        }

                                        @Override
                                        public boolean onError(int code, String error) {
                                            // 接口请求失败
                                            if (mOnTsProgressListener != null) {
                                                mOnTsProgressListener.onFailed(roomCode, extensionId, "接口请求失败");
                                            }
                                            return true;
                                        }
                                    });
                        }
                    }

                    @Override
                    public boolean onError(int code, String error) {
                        if (null != mOnTsProgressListener) {
                            mOnTsProgressListener.onFailed(roomCode, extensionId, error);
                        }
                        return true;
                    }
                });
    }

    private void checkMaxNum(@NonNull String roomCode, String extensionId) {

        mCurPausedList.remove(roomCode);
        mCurPausedList.remove(extensionId);

        // 若发现当前下载数量超过最大下载数量，则暂停正在下载的
        if (mCurDownloadingList != null && mCurDownloadingList.size() >= mMaxDownloadNum) {
            if (mCurDownloadingList.contains(roomCode) || mCurDownloadingList.contains(extensionId)) {

            } else {
                try {
                    // 停止第一个，将现在的添加到最后
                    pause(mCurDownloadingList.get(0));
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                }
            }
        }
    }

    /**
     * 解析服务器文件内容
     */
    private ZGLPlayBackVideoBean processBody(String code, String url, File file) {
        ZGLPlayBackVideoBean newBean = new ZGLPlayBackVideoBean();
        newBean.code = code;

//        String destUrl = url.substring(0, url.lastIndexOf("/"));

        List<String> tmpUrls = new ArrayList<>();
        try {
            BufferedReader bufferedReader = new BufferedReader(new FileReader(file));
            String line = null;
            while ((line = bufferedReader.readLine()) != null) {
                if (!ValidateUtils.isEmpty(line) && line.contains(".ts")) {
//                    tmpUrls.add(destUrl + "/" + line);
                    // 不用再手动拼接下载地址
                    tmpUrls.add(line);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        newBean.urls = tmpUrls;

        return newBean;
    }

    /**
     * 添加新视频的下载
     *
     * @param videoBean
     */
    private synchronized void addNewVideo(String roomName, final String extensionId, final ZGLPlayBackVideoBean videoBean) {
        // 如果已暂停该任务,则不再继续
        final boolean isPaused = mCurPausedList.contains(videoBean.code)
                || mCurPausedList.contains(extensionId);
        if (isPaused) {
            mCurPausedList.remove(videoBean.code);
            mCurPausedList.remove(extensionId);
            return;
        }

        ZGLDownLoaderBean downLoaderBean = new ZGLDownLoaderBean();
        downLoaderBean.setCode(videoBean.code);
        downLoaderBean.setExtensionId(extensionId);
        downLoaderBean.setRoomName(roomName);
        downLoaderBean.setDownloadPath(getDownloadPath(videoBean.code));
        mBeanDao.insertOrReplace(downLoaderBean);

        ThreadPoolUtils.execute(new Runnable() {
            @Override
            public void run() {
                for (int i = 0; i < videoBean.urls.size(); i++) {
                    ZGLDownLoaderFileBean downLoaderFileBean = new ZGLDownLoaderFileBean();
                    downLoaderFileBean.setCode(videoBean.code);
                    downLoaderFileBean.setExtensionId(extensionId);
                    downLoaderFileBean.setDownloadUrl(videoBean.urls.get(i));
                    mFileBeanDao.insertOrReplace(downLoaderFileBean);
                }

                if (mDownloadInfoMap == null) {
                    mDownloadInfoMap = new HashMap<>();
                }

                // 开启下载
                List<ZGLDownLoaderFileBean> tmpList = mFileBeanDao.queryRaw("where CODE=?", videoBean.code);
                if (!(mCurPausedList.contains(videoBean.code)
                        || mCurPausedList.contains(extensionId))) {
                    addTask(videoBean.code, extensionId, tmpList.get(0));
                }

                // 将多个code与对应信息存储到集合中
                mDownloadInfoMap.put(videoBean.code, tmpList);
            }
        });
    }

    private synchronized void addTask(@NonNull final String code, final String extensionId,
                                      final ZGLDownLoaderFileBean fileBean) {

        // 从暂停队列中清除
        mCurPausedList.remove(code);
        mCurPausedList.remove(extensionId);

        // 若发现当前下载数量超过最大下载数量，则暂停正在下载的，并开启新的下载
        checkMaxNum(code, extensionId);

        // 添加到下载队列
        if (TextUtils.isEmpty(extensionId)) {
            if (!mCurDownloadingList.contains(code))
                mCurDownloadingList.add(code);
        } else {
            if (!mCurDownloadingList.contains(extensionId))
                mCurDownloadingList.add(extensionId);
        }


        // 更新DB状态为：下载中
        List<ZGLDownLoaderBean> totalList = mBeanDao.queryRaw("where code=? or EXTENSION_ID=?", code, code);
        if (!ValidateUtils.isEmpty(totalList)) {
            ZGLDownLoaderBean downLoaderBean = totalList.get(0);
            downLoaderBean.setStatus(2);
            mBeanDao.update(downLoaderBean);
        }

        String destPath = getDownloadPath(code) + fileBean.getName();
        File destFile = new File(destPath);
        if (!destFile.exists()) {
            try {
                destFile.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        DownloadFileUtils.with()
                .downloadPath(destPath)
                .url(fileBean.getDownloadUrl())
                .tag(fileBean.getDownloadUrl())
                .execute(new AbsFileProgressCallback() {
                    @Override
                    public void onSuccess(String s) {
                        LogUtils.e(TAG, "下载成功 " + s);
                        fileBean.setStatus(1);
                        mFileBeanDao.update(fileBean);

                        // 判断当前下载完的是否为最后一条。是 则下载完毕，否 则继续下载下一条

                        // 是否为最后一条
                        boolean isLastOne = false;

                        int curPosition = mDownloadInfoMap.get(fileBean.getCode()).indexOf(fileBean);
                        isLastOne = curPosition == mDownloadInfoMap.get(fileBean.getCode()).size() - 1;

                        LogUtils.e(TAG, "totalCount " + mDownloadInfoMap.get(fileBean.getCode()).size() + ", curPosition " + curPosition);

                        if (isLastOne) {
                            // 全部下载完毕

                            LogUtils.e(TAG, fileBean.getCode() + " 全部下载完成");

                            // 更新正在下载的个数
                            if (mCurDownloadingList != null && mCurDownloadingList.size() > 0) {
                                mCurDownloadingList.remove(0);
                            }

                            if (mOnTsProgressListener != null) {
                                mOnTsProgressListener.onSuccess(fileBean.getCode(), extensionId);
                            }

                            // 更新DB状态为：已下载完成
                            List<ZGLDownLoaderBean> queryRaw = mBeanDao.queryRaw("where code=?", fileBean.getCode());
                            if (queryRaw != null && queryRaw.size() != 0) {
                                ZGLDownLoaderBean downLoaderBean = queryRaw.get(0);
                                downLoaderBean.setStatus(1);
                                downLoaderBean.setProgress("100");
                                mBeanDao.update(downLoaderBean);
                            }

                        } else {
                            // 继续下载下一个

                            // 保存当前进度
                            List<ZGLDownLoaderBean> queryRaw = mBeanDao.queryRaw("where code=?", fileBean.getCode());

                            String progress = String.format("%1.2f", (((float) curPosition) / mDownloadInfoMap.get(fileBean.getCode()).size()) * 100);

                            if (queryRaw != null && queryRaw.size() != 0) {
                                ZGLDownLoaderBean downLoaderBean = queryRaw.get(0);
                                if (!TextUtils.isEmpty(progress)
                                        && !TextUtils.isEmpty(downLoaderBean.getProgress())
                                        && Float.parseFloat(progress) < Float.parseFloat(downLoaderBean.getProgress())) {
                                    progress = downLoaderBean.getProgress();
                                }

                                progress = String.valueOf(Math.abs(Float.parseFloat(progress)));
                                downLoaderBean.setProgress(progress);
                                mBeanDao.update(downLoaderBean);

                                LogUtils.e(TAG, "下载进度 " + progress);
                            }

                            // 如果指定房间已经暂停，则不再进行下一个的下载，不再回调进度
                            if (mCurPausedList.contains(fileBean.getCode()) || mCurPausedList.contains(extensionId)) {
                                return;
                            }

                            // 开启下一个的下载
                            List<ZGLDownLoaderFileBean> tmpList = mFileBeanDao.queryRaw("where code=?", fileBean.getCode());
                            if (tmpList != null && tmpList.size() > 0 && curPosition < tmpList.size() - 1) {
                                ZGLDownLoaderFileBean nextBean = tmpList.get(++curPosition);
                                addTask(fileBean.getCode(), extensionId, nextBean);
                            }

                            // 回调进度
                            if (mOnTsProgressListener != null) {
                                mOnTsProgressListener.onProgress(fileBean.getCode(), extensionId, String.valueOf(Math.abs(Float.parseFloat(progress))));
                            }
                        }
                    }

                    @Override
                    public void onProgress(long l, long l1, boolean b) {
                        LogUtils.e(TAG, "下载进度 " + l + "," + l1);
                    }

                    @Override
                    public void onFailed(String s) {
                        LogUtils.e(TAG, "下载失败 " + s);

                        pause(TextUtils.isEmpty(extensionId) ? code : extensionId);

                        if (mOnTsProgressListener != null) {
                            mOnTsProgressListener.onFailed(fileBean.getCode(), extensionId, "下载失败");
                        }

                    }

                    @Override
                    public void onStart() {
                        LogUtils.e(TAG, "下载开始  ");
                        if (mOnTsProgressListener != null) {
                            mOnTsProgressListener.onStart(fileBean.getCode(), extensionId);
                        }
                    }

                    @Override
                    public void onCancle() {
                        LogUtils.e(TAG, "下载取消 ");

                        pause(TextUtils.isEmpty(extensionId) ? code : extensionId);
                    }
                });
    }


    /**
     * 暂停下载
     *
     * @param code 房间口令
     */
    public void pause(@NonNull String code) {
        // 根据房间口令查询到当前正在下载的url ，并取消掉此请求

        if (TextUtils.isEmpty(code)) {
            return;
        }

        // 添加到已暂停队列
        if (!mCurPausedList.contains(code)) {
            mCurPausedList.add(code);
        }
        // 从下载队列中删除
        mCurDownloadingList.remove(code);

        if (mOnTsProgressListener != null) {
            mOnTsProgressListener.onPaused(code, code);
        }

        List<ZGLDownLoaderFileBean> totalFileList = mFileBeanDao.queryRaw("where code=? or EXTENSION_ID=?", code, code);
        List<ZGLDownLoaderBean> totalList = mBeanDao.queryRaw("where code=? or EXTENSION_ID=?", code, code);

        if (!ValidateUtils.isEmpty(totalList)) {
            ZGLDownLoaderBean downLoaderBean = totalList.get(0);
            if (!downLoaderBean.isDownloaded()) {
                downLoaderBean.setStatus(3);
            }
            mBeanDao.update(downLoaderBean);
        }

        if (!ValidateUtils.isEmpty(totalFileList)) {
            int curPos = 0;

            for (int i = 0; i < totalList.size(); i++) {
                if (totalFileList.get(i).getStatus() == 0) {
                    // 第一个未下载的即是当前正在下载的
                    curPos = i;
                    break;
                }
            }

            LogUtils.e(TAG, "暂停 ============= " + curPos);

            DownloadFileUtils.cancle(totalFileList.get(curPos).getDownloadUrl());
        }
    }

    /**
     * 继续下载
     *
     * @param code 房间口令
     */
    private void resumeTask(@NonNull String code, String extensionId) {
        // 根据房间口令查询到当前未下载的第一条开始下载

        List<ZGLDownLoaderFileBean> totalList = mFileBeanDao.queryRaw("where code=? or EXTENSION_ID=?", code, code);

        if (ValidateUtils.isEmpty(totalList)) {
            // 未插入到DB中，从新开始

            // 清除掉主表数据，防止主表中有数据而子表中没数据造成死循环
            mDaoSession.getDatabase().execSQL("delete from ZGLDOWN_LOADER_BEAN where CODE=? or EXTENSION_ID=?", new String[]{"code", "code"});
            mDaoSession.clear();
            start(code);
        } else {

            mDownloadInfoMap.put(code, totalList);

            int curPos = -1;

            for (int i = 0; i < totalList.size(); i++) {
                if (totalList.get(i).getStatus() == 0) {
                    // 第一个未下载的即是当前需要继续下载的
                    curPos = i;
                    break;
                }
            }

            if (curPos == -1) {
                LogUtils.e(TAG, "resumeTask " + code + " 已下载完成");
                if (mOnTsProgressListener != null) {
                    mOnTsProgressListener.onSuccess(code, extensionId);
                }
            } else {
                addTask(code, extensionId, totalList.get(curPos));
            }

        }
    }

    /**
     * 删除全部
     */
    public void deleteAll(final OnZGLDeleteListener onZGLDeleteListener) {
        mBeanDao.deleteAll();
        mFileBeanDao.deleteAll();
        if (onZGLDeleteListener != null) {
            onZGLDeleteListener.onDelete("", true);
        }
        final File dirFile = new File(DOWNLOAD_PATH);
        if (dirFile.exists()) {
            ThreadPoolUtils.execute(new Runnable() {
                @Override
                public void run() {
                    final boolean isDelete = deleteDir(dirFile);
                    LogUtils.e(TAG, "删除SD卡全部内容成功: ");
                }
            });
        }
    }

    /**
     * 删除指定房间的视频文件和DB信息
     *
     * @param code
     */
    public void delete(@NonNull final String code,
                       final OnZGLDeleteListener onZGLDeleteListener) {

        if (TextUtils.isEmpty(code)) {
            return;
        }

        // 先暂停下载，再作删除操作
        pause(code);

        // 删除表数据。因数据量大，采用原生SQL语句
        try {
            mDaoSession.getDatabase().execSQL("delete from ZGLDOWN_LOADER_BEAN where CODE=? or EXTENSION_ID=?", new String[]{"code", "code"});
            mDaoSession.getDatabase().execSQL("delete from ZGLDOWN_LOADER_FILE_BEAN where CODE=? or EXTENSION_ID=?", new String[]{"code", "code"});
        } catch (Exception e) {
            e.printStackTrace();
        }

        // 删除完DB数据，直接回调。避免SD卡内容过多，回调过慢
        if (onZGLDeleteListener != null) {
            onZGLDeleteListener.onDelete(code, true);
        }

        // 删除SD卡内容
        final List<ZGLDownLoaderBean> totalList = mBeanDao.queryRaw("where code=? or EXTENSION_ID=?", code, code);
        if (totalList != null && totalList.size() > 0) {
            for (ZGLDownLoaderBean bean : totalList) {
                mBeanDao.delete(bean);
            }
        }

        try {
            final File dirFile = new File(totalList.get(0).getDownloadPath());
            if (dirFile.exists()) {
                ThreadPoolUtils.execute(new Runnable() {
                    @Override
                    public void run() {
                        final boolean isDelete = deleteDir(dirFile);
                        LogUtils.e(TAG, "删除SD卡内容成功: " + code);
                    }
                });
            }
        } catch (Exception e) {
            e.printStackTrace();
        }


        // 清除DB缓存。否则 mBeanDao mFileBeanDao 查询都会有缓存
        mDaoSession.clear();
    }

    private boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            //递归删除目录中的子目录下
            for (int i = 0; i < children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
        // 目录此时为空，可以删除
        return dir.delete();
    }

    /**
     * 获取当前下载进度
     *
     * @param code
     * @return
     */
    public String getProgress(@NonNull String code) {
        String progress = "0";
        List<ZGLDownLoaderBean> totalList = mBeanDao.queryRaw("where code=? or EXTENSION_ID=?", code, code);
        if (totalList != null && totalList.size() > 0) {
            ZGLDownLoaderBean bean = totalList.get(0);
            progress = bean.getProgress();
        }

        return progress;
    }

    /**
     * 获取全部的下载信息
     *
     * @return
     */
    public List<ZGLDownLoaderBean> getAll() {
        List<ZGLDownLoaderBean> totalList = mBeanDao.loadAll();
        return totalList;
    }

    /**
     * 根据code获取对应视频下载状态
     *
     * @param code
     * @return
     */
    public ZGLDownloadStatusEnum getStatusEnum(@NonNull String code) {
        ZGLDownloadStatusEnum statusEnum = ZGLDownloadStatusEnum.NOT;

        List<ZGLDownLoaderBean> totalList = mBeanDao.queryRaw("where code=? or EXTENSION_ID=?", code, code);
        if (ValidateUtils.isEmpty(totalList)) {
            // DB中不存在此code，说明未下载过
            statusEnum = ZGLDownloadStatusEnum.NOT;
        } else {
            ZGLDownLoaderBean tmpBean = totalList.get(0);
            statusEnum = tmpBean.getStatusEnum();
            if (statusEnum == ZGLDownloadStatusEnum.DOWNLOADED) {
                if (!tmpBean.isLocalM3u8Exist()) {
                    // 此时没有loca.m3u8文件了，则清除DB数据
                    delete(code, null);
                    statusEnum = ZGLDownloadStatusEnum.NOT;
                }
            }
        }
        return statusEnum;
    }


    /**
     * 获取对应视频大小
     *
     * @param code
     * @return
     */
    public long getSize(@NonNull String code) {
        return Long.parseLong(getSize(code, false));
    }

    /**
     * 获取对应视频大小
     *
     * @param code
     * @param format 是否要求格式化，转成 KB MB GB
     * @return
     */
    public String getSize(@NonNull String code, boolean format) {
        String formatSize = "";

        List<ZGLDownLoaderBean> totalList = mBeanDao.queryRaw("where code=? or EXTENSION_ID=?", code, code);
        if (!ValidateUtils.isEmpty(totalList)) {
            ZGLDownLoaderBean tmpBean = totalList.get(0);
            File file = new File(tmpBean.getDownloadPath());
            if (file.exists()) {
                if (!format) {
                    return String.valueOf(ZGLUtils.getFolderSize(file));
                } else {
                    formatSize = ZGLUtils.getFormattedSize(ZGLUtils.getFolderSize(file));
                }
            }
        }

        return formatSize;
    }

    /**
     * 是否已下载成功
     *
     * @param code
     * @return
     */
    public boolean isDownloaded(@NonNull String code) {
        return ZGLDownloadStatusEnum.DOWNLOADED == getStatusEnum(code);
    }

    /**
     * 是否在下载中
     *
     * @param code
     * @return
     */
    public boolean isDownloading(@NonNull String code) {
        return ZGLDownloadStatusEnum.DOWNLOADING == getStatusEnum(code);
    }

    /**
     * 是否在暂停中
     *
     * @param code
     * @return
     */
    public boolean isPaused(@NonNull String code) {
        return ZGLDownloadStatusEnum.PAUSED == getStatusEnum(code);
    }

    /**
     * 是否未下载过
     *
     * @param code
     * @return
     */
    public boolean isDownloadNot(String code) {
        return ZGLDownloadStatusEnum.NOT == getStatusEnum(code);
    }

    /**
     * 将所有下载中的状态重置为已暂停
     */
    public void resetStatusOfDownloading2Paused() {
        List<ZGLDownLoaderBean> list = mBeanDao.loadAll();
        if (list != null && list.size() > 0) {
            for (ZGLDownLoaderBean loaderBean : list) {
                if (loaderBean.isDownloading()) {
                    loaderBean.setStatus(3);
                    mBeanDao.update(loaderBean);
                }
            }
        }
    }

    /**
     * 获取指定房间code的信息
     *
     * @param code
     * @return
     */
    public ZGLDownLoaderBean getBean(@NonNull String code) {
        List<ZGLDownLoaderBean> tmpList = mBeanDao.queryRaw("where CODE=? or EXTENSION_ID=?", code, code);
        if (tmpList != null && tmpList.size() != 0) {
            return tmpList.get(0);
        }

        return null;
    }

    // 下载进度及状态回调监听
    public interface OnZGLProgressListener {

        void onPrepared(String code, String extensionId);

        void onStart(String code, String extensionId);

        void onProgress(String code, String extensionId, String percent);

        void onPaused(String code, String extensionId);

        void onSuccess(String code, String extensionId);

        void onFailed(String code, String extensionId, String message);

    }

    public ZGLDownLoaderManager listener(OnZGLProgressListener onTsProgressListener) {
        mOnTsProgressListener = onTsProgressListener;

        return this;
    }

    private OnZGLProgressListener mOnTsProgressListener;

    // 删除监听
    public interface OnZGLDeleteListener {
        void onDelete(String code, boolean hasDelete);
    }

    public void setOnZGLDeleteListener(OnZGLDeleteListener onZGLDeleteListener) {
        mOnZGLDeleteListener = onZGLDeleteListener;
    }

    private OnZGLDeleteListener mOnZGLDeleteListener;
}
