package com.offcn.live.tsdm.api;


import android.content.Context;

import com.google.gson.Gson;
import com.jyall.base.log.LogUtils;
import com.offcn.live.tsdm.api.LiveApi;

import java.io.IOException;
import java.util.concurrent.TimeUnit;

import io.reactivex.schedulers.Schedulers;
import okhttp3.Headers;
import okhttp3.Interceptor;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.logging.HttpLoggingInterceptor;
import retrofit2.Retrofit;
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory;
import retrofit2.converter.gson.GsonConverterFactory;
import retrofit2.converter.scalars.ScalarsConverterFactory;

/**
 * http 工具类
 *
 * @author wangwenlong
 * @date 2018/7/27
 */

public class RetrofitManager {
    private static final String TAG = RetrofitManager.class.getSimpleName();
    // 超时时间
    private static final long TIME_OUT = 1000 * 30;
    private static LiveApi liveApi;
    private static volatile Retrofit retrofit;
    private static OkHttpClient okHttpClient;

    private static Context mContext;

    public static LiveApi getInstance(Context context) {
        if (retrofit == null) {
            synchronized (RetrofitManager.class) {
                if (retrofit == null) {
                    retrofit = new Retrofit.Builder()
                            .baseUrl(APIConstants.BASE)
                            .addConverterFactory(GsonConverterFactory.create(new Gson()))
                            .addConverterFactory(ScalarsConverterFactory.create())
                            .addCallAdapterFactory(RxJava2CallAdapterFactory.createWithScheduler(Schedulers.io()))
                            .client(getClient())
                            .build();
                    liveApi = retrofit.create(LiveApi.class);
                    mContext = context;
                }
            }
        }
        return liveApi;
    }

    private static OkHttpClient getClient() {
        if (okHttpClient == null) {
            OkHttpClient.Builder okHttpClientBuilder = new OkHttpClient.Builder()
                    .connectTimeout(TIME_OUT, TimeUnit.MILLISECONDS)
                    .readTimeout(TIME_OUT, TimeUnit.MILLISECONDS)
                    .addInterceptor(getCommonHeaderInterceptor())
                    .addInterceptor(getLogInterceptor());

            // 添加 https 信任
            com.offcn.live.api.HttpsCert.SSLParams sslParams = com.offcn.live.api.HttpsCert.getSslSocketFactory(null, null, null);
            okHttpClientBuilder.sslSocketFactory(sslParams.sSLSocketFactory, sslParams.trustManager);

            okHttpClient = okHttpClientBuilder.build();
        }
        return okHttpClient;
    }

    /**
     * 打印 retrofit 日志
     */
    private static HttpLoggingInterceptor getLogInterceptor() {
        return new HttpLoggingInterceptor(new HttpLoggingInterceptor.Logger() {
            @Override
            public void log(String message) {
                LogUtils.e(TAG, "retrofit = " + message);
            }
        }).setLevel(HttpLoggingInterceptor.Level.BODY);
    }

    /**
     * 增加通用的请求头
     */
    private static Interceptor getCommonHeaderInterceptor() {
        return new Interceptor() {
            @Override
            public Response intercept(Chain chain) throws IOException {
                // 增加拦截器，用以加入 http 头部请求信息
                Request.Builder builder = chain.request().newBuilder();
                builder.headers(Headers.of(CommonHeaders.getCommonHeaders(mContext)));
                Request request = builder.build();
                return chain.proceed(request);
            }
        };
    }
}
