package com.offcn.live.tsdm.api.network;

import android.content.Context;
import android.text.TextUtils;

import com.jyall.base.util.CommonUtils;

import io.reactivex.Observer;
import io.reactivex.disposables.Disposable;
import retrofit2.Response;
import tsdm.live.offcn.com.tsdownloadermanager.R;


/**
 * RxJava 请求回调类
 *
 * @author wangwenlong
 * @date 2018/7/27
 */
public abstract class ProgressSubscriber<T> implements Observer<Response<ResponseBean<T>>> {

    private Context mContext;

    private ProgressSubscriber() {
    }

    public ProgressSubscriber(Context context) {
        this.mContext = context;
    }

    /**
     * 不要重写此方法，这个是给Rx调用
     */
    @Override
    @Deprecated
    public void onError(Throwable e) {
        try {
            // 若不重写 onError(int, string) 方法，则直接 toast 提示
            if (!onError(999, e.getMessage())) {
                CommonUtils.showToast(mContext, e.getMessage());
            }
        } catch (Exception exception) {
            CommonUtils.showToast(mContext, R.string.server_error);
        }
    }

    @Override
    public void onComplete() {
    }


    @Override
    public void onSubscribe(Disposable d) {
    }


    @Override
    public void onNext(Response<ResponseBean<T>> response) {
        if (response.errorBody() != null) {
            // 外部错误。若不重写 onError 则直接 toast 错误提示
            if (!onError(response.code(), response.message())) {
                CommonUtils.showToast(mContext, response.message());
            }
        } else {
            // 内部处理
            ResponseBean responseBean = response.body();
            if (responseBean != null) {
                if (responseBean.code == 200) {
                    onResponse((T) responseBean.data);
                } else {
                    // 是否有通用接口返回处理
                    if (!CommonCodeHelper.processCode(responseBean)) {
                        if (!onError(responseBean.code, responseBean.msg)) {
                            if (!TextUtils.isEmpty(responseBean.msg)) {
                                CommonUtils.showToast(mContext, responseBean.msg);
                            }
                        }
                    }
                }
            } else {
                // 返回为空
                onResponse(null);
            }
        }
    }

    /**
     * 请求返回错误
     *
     * @return 如果处理了这个错误信息，返回true， 如果不处理这个错误，而是直接走统一处理（弹Toast），那么就返回false
     */
    public boolean onError(int code, String error) {

        return false;
    }

    public abstract void onResponse(T result);
}
