package com.offcn.live.tsdm.util;

import android.content.Context;

import com.offcn.live.tsdm.bean.ZGLUserBean;


/**
 * 缓存类，比如用户信息，或其它配置
 *
 * @author wangwenlong
 * @date 2018/10/17
 */
public class ZGLCacheManager {

    private static volatile ZGLCacheManager INSTANCE;

    private Context mContext;

    private ZGLCacheManager(Context context) {
        this.mContext = context.getApplicationContext();
    }

    public static ZGLCacheManager getInstance(Context context) {
        if (INSTANCE == null) {
            synchronized (ZGLCacheManager.class) {
                if (INSTANCE == null) {
                    INSTANCE = new ZGLCacheManager(context);
                }
            }
        }
        return INSTANCE;
    }

    // 用户信息
    private ZGLUserBean mUserInfo;


    /**
     * 统一清除缓存方法，退出登录时调用
     */
    public void clear() {
        setUserInfo(null);
        ZGLSPUtils.clear(mContext);
    }

    /**
     * 设置用户信息
     */
    public void setUserInfo(ZGLUserBean userInfo) {
        this.mUserInfo = userInfo;
        ZGLUserInfoHelper.getInstance().saveUserInfo(mContext, userInfo);
    }

    public ZGLUserBean getUserInfo() {
        if (null == mUserInfo) {
            mUserInfo = ZGLUserInfoHelper.getInstance().getUserInfo(mContext);
        }
        return mUserInfo;
    }

}
